'use strict';

// Do this as the first thing so that any code reading it knows the right env.
process.env.BABEL_ENV = 'development';
process.env.NODE_ENV = 'development';

// Makes the script crash on unhandled rejections instead of silently
// ignoring them. In the future, promise rejections that are not handled will
// terminate the Node.js process with a non-zero exit code.
process.on('unhandledRejection', err => {
  throw err;
});

const glob = require('glob');
const bluebird = require('bluebird');
const path = require('path');
const babel = require('babel-core');
const manageTranslations = require('react-intl-translations-manager').default;

const argv = require('./utils/argv');
const paths = require('../config/paths');

const pGlob = bluebird.promisify(glob);

// Ensure environment variables are read.
require('../config/env');

const getMessagesOutputPath = () => path
  .join(paths.appPath, argv.getArgvValue(['--message-output']) || 'messages');

const getTranslationsOutputPath = () => path.join(
  paths.appPath,
  argv.getArgvValue(['--translations-output']) || 'translations'
);

const getLocales = () => {
  const localesString = argv.getArgvValue(['--locales', '-l', '--languages']);
  const locales = localesString ? localesString.split(',') : ['en'];
  return ['default', ...locales];
};


const pTransformFile = bluebird.promisify(babel.transformFile);

// https://github.com/yahoo/babel-plugin-react-intl
const getBabelExtractPlugin = () => [
  require.resolve('babel-plugin-react-intl'),
  {
    "messagesDir": getMessagesOutputPath()
  }
];

// the auto-generating-tool sets the correct name
const getBabelTransformOptions = () => ({
  presets: [
    {
      plugins: [getBabelExtractPlugin()],
    },
    require.resolve('babel-preset-apoly-react-app'),
  ],
  babelrc: false,
});

const printFinished = () => console.log(`finished, messages are extracted into ${getMessagesOutputPath()}, translations are updated in ${getTranslationsOutputPath()}`);

const babelifyWithExtract = file =>
  pTransformFile(file, getBabelTransformOptions());

const extractTranslations = () => manageTranslations({
  messagesDirectory: getMessagesOutputPath(),
  translationsDirectory: getTranslationsOutputPath(),
  languages: getLocales(),
});

pGlob(paths.appSrc + '/**/*.js')
  .then(files => Promise.all(files.map(babelifyWithExtract)))
  .then(extractTranslations)
  .then(printFinished)
  .catch(err => {
    console.log('Failed to compile i18n-files.\n');
    console.log(err);
    console.log(err && err.message);
    process.exit(1);
  });

